/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.item;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.exception.ArianeArithmeticException;
import fr.greyc.pantheon.ariane.model.exception.ArianeRuntimeException;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.presenter.Graph;
import fr.greyc.pantheon.ariane.view.graph.item.ScriptItemView;

import java.io.IOException;
import java.util.List;

/**
 * A <code>ScriptItem</code> implements a long text value.
 */
public final class ScriptItem extends ControlItem {

   public static final String TAGNAME = "Script";

   public ScriptItem( TreeItem treeItem, Graph graph ) {
      super(new ScriptItemView(), treeItem, graph, treeItem.getName());
      setFormula(""); // IMPERATIVE to signify that it uses formula.
      setComment(getFormula());
   }

   public ScriptItem( Graph graph, List<XMLElement> elements ) throws IOException {
      super(new ScriptItemView(), graph);
      setFormula(""); // IMPERATIVE to signify that it uses formula.
      createFromXML(elements);
      setComment(getFormula());
   }

   @Override
   public void setPrototype( String name, int iIm, int iP, int oIm ) {
      setAbstractName(name);
   }

   @Override
   public boolean isDisplayableFormula() {
      return false;
   }

   ///// 
   ///// A C C E S S O R S
   /////
   @Override
   public String getTagName() {
      return TAGNAME;
   }

   ///// 
   ///// E X E C U T I O N
   /////
   @Override
   protected boolean primitiveExecute() throws ArianeRuntimeException {
      try {
         final String formula = getFormula();
         if (formula == null || formula.isEmpty()) {
            throw new ArianeArithmeticException(ArianeSettings.getString("arithmetic.error19"));
         }
         getOutputResultSlot().setValue(formula);
         changeStatus(Status.EXECUTED);
         return true;
      } catch (ArianeArithmeticException e) {
         if (Graph.isExecutionRunning()) {
            Ariane.getPresenter().printErrorMessage(e.getErrorCode(), getAbsoluteName(), getId(), e.getLocalizedMessage());
            getGraph().stopExecution();
            changeStatus(Status.ERROR);
         }
         throw new ArianeRuntimeException(e);
      }
   }
}
