/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.item.etc;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.PlatformSettings;
import java.io.File;
import java.util.List;
import java.util.Map;

/**
 * <code>OperatingSystemProcess</code> redefines Java ProcessBuilder in order
 * to add some variables in the execution environment.
 * It is an adaptor (in the sense of Design Pattern) of the class ProcessBuilder
 * since ProcessBuilder is a final class.
 */
public abstract class OperatingSystemProcess {

   private ProcessBuilder _processBuilder;

   public void createProcess( List<String> commandLine ) {
      _processBuilder = new ProcessBuilder(commandLine);
      setEnvironment();
   }

   public void createProcess( String commandLine ) {
      _processBuilder = new ProcessBuilder(commandLine);
      setEnvironment();
   }

   /**
    * Sets the convenient execution environment for both UNIX,
    * Windows and MacOSX (PATH and LD_LIBRARY_PATH).
    */
   protected final void setEnvironment() {
      _processBuilder.redirectErrorStream(true);
      Map<String, String> env = _processBuilder.environment();
      env.put("PATH", ArianeSettings.getLibraryPath() + File.separatorChar + "bin" + env.get("PATH"));
      String propertyName = PlatformSettings.getLibraryPathPropertyName();
      env.put(propertyName, env.get(propertyName) + File.pathSeparatorChar + ArianeSettings.getLibraryPath() + File.separatorChar + "lib");
   }

   protected final ProcessBuilder getProcessBuilder() {
      return _processBuilder;
   }
}
