/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.slot;

import java.awt.Point;
import java.awt.geom.Line2D;
import java.util.ArrayList;

/**
 * A <code>ValueLink</code> is a connection between two slots
 * that supports values. It is represented as a polyline.
 */
public final class ValueLink extends Link {

   public ValueLink( InputSlot slot ) {
      super(slot);
   }

   /**
    * Builds the polyline that corresponds to the link.
    *
    * @return the series of straight lines.
    */
   @Override
   protected ArrayList<Point> createPolyline() {
      ArrayList<Point> path = new ArrayList<>();

      // Insert the first point.
      path.add(0, new Point(getSourceCoordinate()));

      final int[] xcoord = new int[6];
      final int[] ycoord = new int[6];

      xcoord[3] = (int) getTargetCoordinate().getX();
      ycoord[3] = (int) getTargetCoordinate().getY();
      xcoord[0] = (int) getSourceCoordinate().getX();
      ycoord[0] = (int) getSourceCoordinate().getY();

      if (ycoord[0] <= ycoord[3]) { // nominal case: item 1 below item 2
         ycoord[1] = ycoord[3] - (ycoord[3] - ycoord[0]) / 2;
         xcoord[1] = xcoord[0];
         ycoord[2] = ycoord[1];
         xcoord[2] = xcoord[3];

         path.add(new Point(xcoord[1], ycoord[1]));
         path.add(new Point(xcoord[2], ycoord[2]));
      } else { //degenerate case... item 2 above item 1
         ycoord[5] = ycoord[3];
         xcoord[5] = xcoord[3];
         ycoord[1] = ycoord[0] + 10;
         xcoord[1] = xcoord[0];
         if (xcoord[0] < xcoord[5] - 15) { // link in front  of element 1.
            ycoord[2] = ycoord[1];
            xcoord[2] = (xcoord[5] - 10 - xcoord[0]) / 2 + xcoord[0];
            ycoord[3] = ycoord[5] - 10;
            xcoord[3] = xcoord[2];
         } else { // link behind item 1.
            ycoord[2] = ycoord[0] + 10;
            xcoord[2] = xcoord[5] + 20;
            ycoord[3] = ycoord[5] - 10;
            xcoord[3] = xcoord[2];
         }
         ycoord[4] = ycoord[3];
         xcoord[4] = xcoord[5];

         path.add(new Point(xcoord[1], ycoord[1]));
         path.add(new Point(xcoord[2], ycoord[2]));
         path.add(new Point(xcoord[3], ycoord[3]));
         path.add(new Point(xcoord[4], ycoord[4]));
      }
      // insert the last point.
      path.add(new Point(getTargetCoordinate()));

      return path;
   }

   /**
    * Moves the current selected intermediate line of the link according to the new coordinates.
    *
    * @param xycoord the new coordinates.
    */
   @Override
   public void move( Point xycoord ) {
      setUserDefined(true);

      int selectedLine = getSelectedLine();
      if (selectedLine < 1) {
         return;
      }
      Line2D.Float[] polyline = getPolyline();
      if (polyline.length == 3) {
         float y;
         if (xycoord.y < polyline[0].y1 + GAP) {
            y = polyline[0].y1 + GAP;
         } else if (xycoord.y > polyline[polyline.length - 1].y2 - GAP) {
            y = polyline[polyline.length - 1].y2 - GAP;
         } else {
            y = xycoord.y;
         }
         polyline[0].y2 = y;
         polyline[1].y1 = y;
         polyline[1].y2 = y;
         polyline[2].y1 = y;
      } else {
         switch (selectedLine) {
            case 1: {
               float y;
               if (xycoord.y < polyline[0].y1 + GAP) {
                  y = polyline[0].y1 + GAP;
               } else {
                  y = xycoord.y;
               }
               polyline[0].y2 = y;
               polyline[1].y1 = y;
               polyline[1].y2 = y;
               polyline[2].y1 = y;
            }
            break;
            case 2: {
               float x;
               if (xycoord.x < polyline[0].x1 + GAP) {
                  x = polyline[0].x1 + GAP;
               } else if (xycoord.x > polyline[polyline.length - 1].x2 - GAP) {
                  x = polyline[polyline.length - 1].x2 - GAP;
               } else {
                  x = xycoord.x;
               }
               polyline[1].x2 = x;
               polyline[2].x1 = x;
               polyline[2].x2 = x;
               polyline[3].x1 = x;
            }
            break;
            case 3: {
               float y;
               if (xycoord.y > polyline[polyline.length - 1].y2 - GAP) {
                  y = polyline[polyline.length - 1].y2 - GAP;
               } else {
                  y = xycoord.y;
               }
               polyline[4].y1 = y;
               polyline[3].y1 = y;
               polyline[3].y2 = y;
               polyline[2].y2 = y;
            }
            break;
            default:
               break;
         }
      }
   }

   /**
    * Modifies the link because one the source or target items has been moved.
    *
    * @return false if the link cannot be modified because it becomes too complex.
    */
   @Override
   protected boolean modifyPolyline() {
      double sourceY = getSourceCoordinate().getY();
      double targetY = getTargetCoordinate().getY();
      Line2D.Float[] polyline = getPolyline();
      if (polyline.length == 3 && sourceY >= targetY || polyline.length == 5 && sourceY < targetY) {
         return false;
      }

      double sourceX = getSourceCoordinate().getX();
      double targetX = getTargetCoordinate().getX();
      if (sourceY != polyline[0].y1 || sourceX != polyline[0].x1) {
         polyline[0].x1 = (float) sourceX;
         polyline[0].x2 = (float) sourceX;
         polyline[1].x1 = (float) sourceX;
         polyline[0].y1 = (float) sourceY;
         if (sourceY > polyline[0].y2) {
            return false;
         }
      } else if (targetY != polyline[polyline.length - 1].y1 || targetX != polyline[polyline.length - 1].x1) {
         polyline[polyline.length - 1].x2 = (float) targetX;
         polyline[polyline.length - 1].x1 = (float) targetX;
         polyline[polyline.length - 2].x2 = (float) targetX;
         polyline[polyline.length - 1].y2 = (float) targetY;
         if (targetY < polyline[polyline.length - 1].y1) {
            return false;
         }
      }
      return true;
   }
}
