/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.presenter;

import java.io.File;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.graph.GraphModel;
import fr.greyc.pantheon.ariane.model.graph.item.GraphItem;

/**
 * A <code>RootGraph</code> defines a program.
 */
public final class RootGraph extends Graph {

   private boolean _isModified;

   public RootGraph( String name ) {
      setName(name);
   }

   public RootGraph( File file ) {
      super(file);
      setName(getFileBasename(file.getName()));
   }

   public RootGraph( String name, GraphModel graphModel ) { // For tests
      super(graphModel);
      setName(name);
   }

   @Override
   public GraphItem getSuperGraphItem() {
      return null;
   }

   @Override
   public RootGraph getRoot() {
      return this;
   }

   @Override
   public boolean isRoot() {
      return true;
   }

   @Override
   public boolean isModified() {
      return _isModified;
   }

   @Override
   public void setModified( boolean value ) {
      if (isAlive()) {
         Ariane.getPresenter().setModifiedSaveB(value);
         _isModified = value;
      }
   }

   @Override
   public boolean close() {
      if (!isModified() || isCloseConfirmed()) {
         closeFrame();
         dispose();
         Ariane.getPresenter().updateRecentFilesListMenu();
         return true;
      } else {
         return false;
      }
   }

   @Override
   public void save() {
      File file = getStorageFile();
      boolean isFirstSave = file == null;
      if (isFirstSave) {
         saveAs();
      } else {
         save(getStorageFile());
      }
   }

   private String getFileBasename( String filename ) {
      return filename.substring(0, filename.lastIndexOf('.'));
   }

   @Override
   public void saveAs() {
      File theFile = askForFilename();
      if (theFile != null) {
         save(theFile);
         setName(getFileBasename(theFile.getName()));
         renameFrame();
         Ariane.getPresenter().addRecentProjects(theFile.getAbsolutePath());
      }
   }
}
