/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.presenter.command;

import javax.swing.JComponent;
import javax.swing.undo.AbstractUndoableEdit;

import fr.greyc.pantheon.ariane.model.graph.item.GraphItem;

/**
 * This class is a part of the history manager.
 * It stores a change binary value action.
 */
public final class ChangeBinaryEdit extends AbstractUndoableEdit {

   private final GraphItem _graphItem;
   private final JComponent _textfield;
   private final String _newValue;
   private final String _oldValue;

   public ChangeBinaryEdit( GraphItem graphItem, JComponent textfield, String newValue, String oldValue ) {
      _graphItem = graphItem;
      _textfield = textfield;
      _newValue = newValue;
      _oldValue = oldValue;
   }

   @Override
   public void redo() {
      _graphItem.getView().requestFocusInWindow();
      _graphItem.changeBinary(_textfield, _newValue, false);
   }

   @Override
   public void undo() {
      _graphItem.getView().requestFocusInWindow();
      _graphItem.changeBinary(_textfield, _oldValue, false);
   }
}
