/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view;

import fr.greyc.pantheon.ariane.ArianeSettings;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;

import com.sun.management.UnixOperatingSystemMXBean;

import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;
import java.text.DecimalFormat;

/**
 * A <tt>AboutBox</tt> displays general information about Ariane
 * such as authors, version number, memory statistics, etc.
 */
public final class AboutBox extends JDialog {

   public AboutBox( final JFrame parent ) {
      super(parent, ArianeSettings.getString("AboutBox.title"), true);
      final JTabbedPane tabbedPane = new JTabbedPane();

      // Tab 1: Logo
      {
         final JPanel panel = new JPanel();
         final Box panel1 = Box.createVerticalBox();
         panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("AboutBox.version.number")));
         panel1.add(new JLabel(ArianeSettings.VERSION_ID, JLabel.CENTER));
         panel1.add(Box.createVerticalStrut(10));
         panel1.add(new JLabel(ArianeSettings.getIcon("splash.png")), BorderLayout.CENTER);
         panel1.add(Box.createVerticalStrut(10));
         panel1.add(Box.createGlue());
         panel.add(panel1);
         tabbedPane.addTab(ArianeSettings.getString("AboutBox.version.title"), panel);
      }
      super.getContentPane().add(tabbedPane);
      super.pack();
      super.setResizable(false);

      // Tab 2: version
      {
         final Box panel = Box.createVerticalBox();

         final JPanel panel1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
         panel.add(panel1);
         panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("AboutBox.warning")));
         final String text = ArianeSettings.getString("AboutBox.license.text");
         final JTextArea textArea = new JTextArea(text);
         final Font font = new Font("Serif", Font.PLAIN, 13);
         textArea.setFont(font);
         textArea.setEditable(false);
         panel1.add(textArea);
         tabbedPane.addTab(ArianeSettings.getString("AboutBox.license.title"), panel);
      }

      // Tab 3 authors
      {
         final Box panel = Box.createVerticalBox();
         final JPanel panel1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
         panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("AboutBox.contribution")));
         final Box box1 = Box.createVerticalBox();
         for (String name : ArianeSettings.AUTHORS) {
            box1.add(new JLabel(" - " + name));
         }
         panel1.add(box1);
         panel.add(panel1);

         final JPanel panel2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
         panel2.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("AboutBox.addressTitle")));
         final Box box2 = Box.createVerticalBox();
         box2.add(new JLabel(ArianeSettings.getString("AboutBox.team")));
         box2.add(new JLabel(ArianeSettings.getString("AboutBox.lab")));
         box2.add(new JLabel(ArianeSettings.getString("AboutBox.address")));
         box2.add(new JLabel(ArianeSettings.getString("AboutBox.zipcode")));
         box2.add(new JLabel(ArianeSettings.getString("AboutBox.http")));
         panel2.add(box2);
         panel.add(panel2);
         tabbedPane.addTab(ArianeSettings.getString("AboutBox.authors.title"), panel);
      }

      // Tab 4 statistics
      {
         final Box panel = Box.createVerticalBox();
         System.gc();
         JPanel statics;
         final DecimalFormat format = new DecimalFormat("#.##");
         double stat;

         statics = new JPanel(new FlowLayout(FlowLayout.LEFT));
         statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("AboutBox.statistics.usedMemory")));
         stat = Runtime.getRuntime().totalMemory() / (1024.0 * 1024.0);
         statics.add(new JLabel(ArianeSettings.getString("AboutBox.statistics.size") + " " + format.format(stat) + ArianeSettings.getString("AboutBox.statistics.megabytes")));
         panel.add(statics);

         statics = new JPanel(new FlowLayout(FlowLayout.LEFT));
         statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("AboutBox.statistics.usedPercent")));
         final double percent = ((double) Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()) / Runtime.getRuntime().maxMemory();
         statics.add(new JLabel(ArianeSettings.getString("AboutBox.statistics.size") + " " + format.format(percent) + "%"));
         panel.add(statics);

         statics = new JPanel(new FlowLayout(FlowLayout.LEFT));
         statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("AboutBox.statistics.availableProcessors")));
         statics.add(new JLabel(ArianeSettings.getString("AboutBox.statistics.number") + " " + Runtime.getRuntime().availableProcessors()));
         panel.add(statics);

         final OperatingSystemMXBean operatingSystemMXBean = ManagementFactory.getOperatingSystemMXBean();

         if (operatingSystemMXBean instanceof UnixOperatingSystemMXBean) {
            final UnixOperatingSystemMXBean operatingSystem = (UnixOperatingSystemMXBean) operatingSystemMXBean;
            statics = new JPanel(new FlowLayout(FlowLayout.LEFT));
            statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("AboutBox.statistics.maxFileDescriptorCount")));
            statics.add(new JLabel(ArianeSettings.getString("AboutBox.statistics.number") + " " + operatingSystem.getMaxFileDescriptorCount()));
            panel.add(statics);

            statics = new JPanel(new FlowLayout(FlowLayout.LEFT));
            statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("AboutBox.statistics.openFileDescriptorCount")));
            statics.add(new JLabel(ArianeSettings.getString("AboutBox.statistics.number") + " " + operatingSystem.getOpenFileDescriptorCount()));
            panel.add(statics);
         }

         tabbedPane.addTab(ArianeSettings.getString("AboutBox.statistics.title"), panel);
      }

      // Tab 5: Legend.
      {
         final Box panel = Box.createVerticalBox();

         final JPanel panel1 = new JPanel(new BorderLayout());
         panel.add(panel1);
         panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("AboutBox.legend.header")));
         final String text = ArianeSettings.getString("AboutBox.legend.text");
         final JEditorPane textArea = new JEditorPane("text/html", text);
         textArea.setEditable(false);
         panel1.add(textArea, BorderLayout.CENTER);
         tabbedPane.addTab(ArianeSettings.getString("AboutBox.legend.title"), panel);
      }
      super.setLocationRelativeTo(parent);
   }
}
