/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.WindowConstants;
import javax.swing.UIManager.LookAndFeelInfo;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.presenter.Presenter;

/**
 * This frame is the main frame that supports all the software.
 */
public final class ArianeFrame extends JFrame {

   private Dimension _size;

   public void initFrame( final Presenter presenter ) {
      final Workspace workspace = presenter.getWorkspace();
      setContentPane(workspace);
      setTitle(ArianeSettings.FRAME_TITLE);
      addWindowListener(new WindowAdapter() {
         @Override
         public void windowClosing( WindowEvent event ) {
            if (workspace.isQuitConfirmed()) {
               ((JFrame) (event.getWindow())).setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
               System.exit(0);
            } else {
               ((JFrame) (event.getWindow())).setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
            }
         }
      });
   }

   public Dimension getFinalSize() {
      if (getExtendedState() == Frame.MAXIMIZED_BOTH) {
         return _size;
      } else {
         return super.getSize();
      }
   }

   public void restoreFrameSize() {
      _size = UserPreferences.getFrameSize();
      if (_size != null) {
         setPreferredSize(_size);
         setSize(_size);
      }
      if (UserPreferences.isFrameMaximized()) {
         setExtendedState(Frame.MAXIMIZED_BOTH);
      }
      setLocationByPlatform(true);
   }

   public void installUI() {
      try {
         for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
            if ("Nimbus".equals(info.getName())) {
               UIManager.setLookAndFeel(info.getClassName());
               break;
            }
         }
         UIManager.put("control", new Color(215, 214, 223));
         UIManager.put("nimbusBlueGrey", new Color(167, 167, 190));
         UIManager.put("nimbusBase", new Color(42, 42, 140));
         UIManager.put("TextField.disabledText", Color.BLACK);
         UIManager.put("TextField[Disabled].textForeground", Color.BLACK);
         UIManager.put("TextField[Disabled].borderPainter", UIManager.get("TextField[Enabled].borderPainter"));
         UIManager.put("TabbedPane:TabbedPaneTab[Pressed+Selected].backgroundPainter", UIManager.get("TabbedPane:TabbedPaneTab[Selected].backgroundPainter"));
         UIManager.put("TabbedPane:TabbedPaneTab[Focused+Pressed+Selected].backgroundPainter", UIManager.get("TabbedPane:TabbedPaneTab[Selected].backgroundPainter"));
      } catch (ReflectiveOperationException | UnsupportedLookAndFeelException e) {
         System.err.println("ERREUR");
      }
   }
}
