/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.swingext.GDialog;
import fr.greyc.pantheon.swingext.MessageBox;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.Locale;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultHighlighter;
import javax.swing.text.Document;
import javax.swing.text.Highlighter;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

/**
 * The <code>Console</code> is used to display styled text.
 */
public final class Console extends JTextPane {
   //	private final WordSearcher _searcher;

   private PrintStream _printOutputStream;
   private ReadWriteThread _outputReadWriteThread;

   public Console() {
      //		_searcher = new WordSearcher(this);
      addMouseListener(new MouseAdapter() {
         private void showPopupMenu( final MouseEvent mouseEvent ) {
            JPopupMenu popup = new JPopupMenu();
            JMenuItem erase = new JMenuItem(ArianeSettings.getString("Workspace.console.clearMenuText"));
            erase.addActionListener(new ActionListener() {
               @Override
               public void actionPerformed( ActionEvent event ) {
                  clear();
               }
            });
            JMenuItem search = new JMenuItem(ArianeSettings.getString("Workspace.console.searchMenuText"));
            search.addActionListener(new ActionListener() {
               @Override
               public void actionPerformed( ActionEvent event ) {
                  search(mouseEvent.getLocationOnScreen());
               }
            });
            JMenuItem save = new JMenuItem(ArianeSettings.getString("Workspace.console.saveMenuText"));
            save.addActionListener(new ActionListener() {
               @Override
               public void actionPerformed( ActionEvent event ) {
                  save();
               }
            });
            popup.add(erase);
            popup.add(search);
            popup.add(save);
            popup.show((Component) mouseEvent.getSource(), mouseEvent.getX(), mouseEvent.getY());
         }

         @Override
         public void mousePressed( MouseEvent event ) {
            if (event.isPopupTrigger()) {
               showPopupMenu(event);
            }
         }

         @Override
         public void mouseReleased( MouseEvent event ) {
            if (event.isPopupTrigger()) {
               showPopupMenu(event);
            }
         }
      });

      setEditable(false);
      setDocument(new HTMLDocument());
      setEditorKit(new HTMLEditorKit());
      redirectStreams();
   }

   public void clear() {
      final HTMLDocument document = (HTMLDocument) Console.this.getDocument();
      try {
         document.remove(0, document.getLength());
      } catch (BadLocationException e) {
      }
   }

   private void search( Point mouseCoordinates ) {
      final GDialog dialog = new GDialog(Ariane.getFrame(), ArianeSettings.getString("Workspace.console.searchTitle"), false);
      dialog.setLayout(new BorderLayout(1, 1));
      dialog.setAlwaysOnTop(true);

      JPanel content = new JPanel();
      final JTextField textfield = new JTextField(10);
      content.add(new JLabel(ArianeSettings.getString("Workspace.console.searchTitle")));
      content.add(textfield);
      dialog.add(content, BorderLayout.CENTER);

      JButton okButton = new JButton(ArianeSettings.getString("Workspace.console.findButton"));
      okButton.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            String sentence = textfield.getText().trim();
            int offset = searchAndHighlight(sentence);
            if (offset != -1) {
               try {
                  Console.this.scrollRectToVisible(Console.this.modelToView(offset));
               } catch (BadLocationException exception) {
               }
            }
            dialog.dispose();
         }
      });
      JButton cancelButton = new JButton(ArianeSettings.getString("cancelButtonText"));
      cancelButton.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            dialog.dispose();
         }
      });
      JPanel buttonPanel = new JPanel();
      buttonPanel.add(okButton);
      buttonPanel.add(cancelButton);
      dialog.add(buttonPanel, BorderLayout.SOUTH);
      dialog.pack();
      dialog.setLocation(mouseCoordinates);
      dialog.setVisible(true);
      dialog.setDefaultButton(okButton);
      dialog.setCancelButton(cancelButton);
   }

   private void save() {
      String currentDir = UserPreferences.getCurrentDirectory();
      File currentFileName = new File(currentDir);
      JFileChooser fileChooser = new JFileChooser(currentFileName);
      fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
      fileChooser.setDialogTitle(ArianeSettings.getString("FileChooser.titleSaveLog"));

      if (fileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
         File theFile = fileChooser.getSelectedFile();
         if (isSavingConfirmed(theFile)) {
            writeIntoFile(theFile);
         }
      }
   }

   private boolean isSavingConfirmed( File theFile ) {
      if (theFile != null && theFile.exists()) {
         String[] options = {ArianeSettings.getString("overwriteButtonText"), ArianeSettings.getString("nooverwriteButtonText")};
         if (MessageBox.showConfirmDialog(Ariane.getFrame(),
             ArianeSettings.getString("Workspace.dialog.message.part1")
             + theFile.getName()
             + ArianeSettings.getString("Workspace.dialog.message.part2"),
             ArianeSettings.getString("Workspace.dialog.title"),
             options, options[0]) == 0) {
            return true;
         }
      }
      return false;
   }

   private void writeIntoFile( File theFile ) {
      PrintWriter out = null;
      try {
         String text = getDocument().getText(0, getDocument().getLength());
         out = new PrintWriter(new BufferedWriter(new FileWriter(theFile)));
         out.print(text);
         Ariane.getPresenter().printSystemMessage(theFile + ArianeSettings.getString("Workspace.saving.success"));
      } catch (IOException | BadLocationException ex) {
         Ariane.getPresenter().printErrorMessage(ArianeSettings.getString("Workspace.saving.error1") + getName() + " " + ex.getLocalizedMessage());
      } finally {
         if (out != null) {
            out.close();
         }
      }
   }

   protected int searchAndHighlight( String sentence ) {
      Highlighter highlighter = getHighlighter();
      highlighter.removeAllHighlights();
      if (sentence.isEmpty()) {
         return -1;
      }

      String content;
      try {
         Document document = getDocument();
         content = document.getText(0, document.getLength()).toLowerCase();
      } catch (BadLocationException e) {
         return -1;
      }
      String lowerCaseWord = sentence.toLowerCase(Locale.US);
      int lastIndex = 0;
      int wordSize = lowerCaseWord.length();
      int firstOffset = -1;
      Highlighter.HighlightPainter painter = new DefaultHighlighter.DefaultHighlightPainter(Color.LIGHT_GRAY);
      while ((lastIndex = content.indexOf(lowerCaseWord, lastIndex)) != -1) {
         int endIndex = lastIndex + wordSize;
         try {
            highlighter.addHighlight(lastIndex, endIndex, painter);
         } catch (BadLocationException e) {

         }
         if (firstOffset == -1) {
            firstOffset = lastIndex;
         }
         lastIndex = endIndex;
      }
      return firstOffset;
   }

   private void displayMessage( String text, String color ) {
      int maxLength = 50;
      String printedText = "<font color='" + color + "'>" + text + "</font>";
      while (printedText.length() > maxLength) {
         _printOutputStream.print(printedText.substring(0, maxLength));
         _printOutputStream.flush();
         synchronized (_outputReadWriteThread) {
            _outputReadWriteThread.notifyAll();
         }
         printedText = printedText.substring(maxLength);
      }
      _printOutputStream.println(printedText);
      _printOutputStream.flush();
      synchronized (_outputReadWriteThread) {
         _outputReadWriteThread.notifyAll();
      }
   }

   public void displayOutputMessage( String text ) {
      displayMessage(text, "Blue");
   }

   public void displayErrorMessage( String text ) {
      displayMessage(text, "Red");
   }

   public void displaySystemMessage( String text ) {
      displayMessage(text, "Black");
   }

   private void redirectStreams() {
      try {
         PipedInputStream piCustom = new PipedInputStream();
         PipedOutputStream poCustom = new PipedOutputStream(piCustom);
         _printOutputStream = new PrintStream(poCustom, false);
         _outputReadWriteThread = new ReadWriteThread(piCustom);
         _outputReadWriteThread.start();
      } catch (IOException e) {
      }
   }

   final class ReadWriteThread extends Thread {

      private final PipedInputStream _pipedInputStream;

      ReadWriteThread( PipedInputStream pi ) {
         super();
         _pipedInputStream = pi;
      }

      @Override
      public void run() {
         final HTMLDocument doc = (HTMLDocument) Console.this.getDocument();
         final BufferedReader br = new BufferedReader(new InputStreamReader(_pipedInputStream));
         final HTMLEditorKit kit = (HTMLEditorKit) Console.this.getEditorKit();
         while (true) {
            try {
               String line;
               while (_pipedInputStream.available() > 0) {
                  while ((line = br.readLine()) != null) {
                     try {
                        synchronized (doc) {
                           if (doc.getLength() > 100000) {
                              doc.remove(0, line.length());
                           }
                           kit.insertHTML(doc, doc.getLength(), line + "\n", 0, 0, null);
                           Console.this.setCaretPosition(Console.this.getDocument().getLength());
                        }
                        sleep(10);
                     } catch (BadLocationException e) {
                     }
                  }
               }
               synchronized (this) {
                  while (_pipedInputStream.available() == 0) {
                     wait(); // Do not occupy the CPU no more
                  }
               }
            } catch (IOException | InterruptedException e) {
            }
         }
      }
   }
}
