/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.swingext.GDialog;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * The class <code>OperatorLibraryPathSetting</code> defines a dialog box
 * where the user can set the operator library path.
 */
public final class OperatorLibraryPathChooser extends GDialog {

   private String _operatorLibraryPath;
   private boolean _useOperatorLibrary;
   private boolean _validated;
   private JTextField _textfield2;
   private JButton _browser;
   private JLabel _label;

   public OperatorLibraryPathChooser( boolean isUsed, String path ) {
      super((JFrame) null, ArianeSettings.getString("LibraryPathSetting.title"), true);
      _useOperatorLibrary = isUsed;
      _operatorLibraryPath = path;

      setResizable(false);
      displayContent();
      changeState(_useOperatorLibrary);
      setLocationRelativeTo(null);
      addWindowListener(new WindowAdapter() {
         @Override
         public void windowClosing( WindowEvent event ) {
            closeDialogWithStatus(false);
         }
      });
   }

   public boolean isValidated() {
      return _validated;
   }

   public String getOperatorLibraryPath() {
      return _operatorLibraryPath;
   }

   public boolean isUsed() {
      return _useOperatorLibrary;
   }

   private void displayContent() {
      JPanel panel = new JPanel();
      panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
      panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("LibraryPathSetting.libraryPathName")));

      panel.add(createCheckBox(), JPanel.LEFT_ALIGNMENT);
      JPanel subpanel = new JPanel(new GridLayout(2, 1));
      subpanel.setAlignmentX(LEFT_ALIGNMENT);
      subpanel.setBorder(BorderFactory.createEmptyBorder(0, 40, 0, 0));
      subpanel.add(createLabel());
      subpanel.add(createFileChooser());
      panel.add(subpanel);

      JPanel buttonPanel = new JPanel();
      JButton applyButton = createApplyButton();
      JButton cancelButton = createCancelButton();
      buttonPanel.add(applyButton);
      buttonPanel.add(cancelButton);

      getContentPane().setLayout(new BorderLayout());
      getContentPane().add(panel, BorderLayout.CENTER);
      getContentPane().add(buttonPanel, BorderLayout.SOUTH);

      setDefaultButton(applyButton);
      setCancelButton(cancelButton);
      _textfield2.requestFocusInWindow();

      pack();
   }

   private JLabel createLabel() {
      _label = new JLabel(ArianeSettings.getString("LibraryPathSetting.libraryPathDescription"));
      return _label;
   }

   private JPanel createFileChooser() {
      _textfield2 = new JTextField(30);
      _textfield2.setText(_operatorLibraryPath);
      _textfield2.setCaretPosition(0);
      _textfield2.moveCaretPosition(_operatorLibraryPath.length());
      _textfield2.addFocusListener(new FocusAdapter() {
         @Override
         public void focusLost( FocusEvent event ) {
            _operatorLibraryPath = ((JTextField) event.getSource()).getText();
         }
      });

      _browser = new JButton(ArianeSettings.getString("Ariane.browse"));
      _browser.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            final JFileChooser fc2 = new JFileChooser();
            fc2.setCurrentDirectory(new File(ArianeSettings.getLibraryPath()));
            fc2.setDialogType(JFileChooser.OPEN_DIALOG);
            fc2.setDialogTitle(ArianeSettings.getString("FileChooser.titleSelectDir"));
            fc2.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            int returnVal = fc2.showOpenDialog(OperatorLibraryPathChooser.this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
               _operatorLibraryPath = fc2.getSelectedFile().getAbsolutePath();
               _textfield2.setText(_operatorLibraryPath);
               File file = new File(_operatorLibraryPath + ArianeSettings.getLibraryConfigFilename());
               if (!file.exists()) {
                  JOptionPane.showMessageDialog(OperatorLibraryPathChooser.this,
                      ArianeSettings.getString("LibraryPathSetting.alertMessage"),
                      ArianeSettings.getString("LibraryPathSetting.alertTitle"),
                      JOptionPane.ERROR_MESSAGE);
               }
            }
         }
      });
      JPanel panel = new JPanel();
      panel.add(_textfield2);
      panel.add(_browser);
      return panel;
   }

   private JCheckBox createCheckBox() {
      final JCheckBox checkBox = new JCheckBox(ArianeSettings.getString("LibraryPathSetting.useOperatorLibrary"), UserPreferences.isOperatorLibraryUsed());
      checkBox.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            _useOperatorLibrary = ((JCheckBox) event.getSource()).isSelected();
            changeState(_useOperatorLibrary);
         }
      });
      return checkBox;
   }

   private JButton createCancelButton() {
      JButton cancelButton = new JButton();
      Action cancelKeyAction = new AbstractAction(ArianeSettings.getString("LibraryPathSetting.cancel"), ArianeSettings.getIcon("gtk-cancel.png")) {
         @Override
         public void actionPerformed( ActionEvent event ) {
            closeDialogWithStatus(false);
         }
      };
      cancelButton.setAction(cancelKeyAction);
      return cancelButton;
   }

   private JButton createApplyButton() {
      JButton applyButton = new JButton(ArianeSettings.getString("LibraryPathSetting.apply"), ArianeSettings.getIcon("gtk-apply.png"));
      applyButton.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            closeDialogWithStatus(true);
         }
      });
      return applyButton;
   }

   private void changeState( boolean state ) {
      _textfield2.setEnabled(state);
      _browser.setEnabled(state);
      _label.setEnabled(state);
   }

   private void closeDialogWithStatus( boolean state ) {
      _validated = state;
      setVisible(false);
      dispose();
   }
}
