/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view;

import fr.greyc.pantheon.ariane.ArianeSettings;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JWindow;
import javax.swing.border.EtchedBorder;

/**
 * A <code>SplashScreen</code> displays a given image
 * during the initialization of the software.
 */
public final class SplashScreen extends JWindow {

   private final ProgressBar _progressBar = new ProgressBar();

   public SplashScreen( final ImageIcon splashImage ) {
      final JLabel splashButton = new JLabel("");
      if (splashImage != null) {
         final int imgWidth = splashImage.getIconWidth();
         final int imgHeight = splashImage.getIconHeight();
         final Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize();
         setLocation(scrSize.width / 2 - imgWidth / 2, scrSize.height / 2 - imgHeight / 2);
         splashButton.setIcon(addReleaseNumberOntoImage(splashImage));
      }
      setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      getContentPane().setLayout(new BorderLayout(0, 0));
      getContentPane().add(splashButton, BorderLayout.CENTER);
      getContentPane().add(_progressBar, BorderLayout.SOUTH);
      setAlwaysOnTop(true);
      pack();
   }

   private ImageIcon addReleaseNumberOntoImage( final ImageIcon imageIcon ) {
      final Image image = imageIcon.getImage();
      final int width = image.getWidth(null);
      final int height = image.getHeight(null);
      final BufferedImage bimg = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
      bimg.createGraphics().drawImage(image, 0, 0, this);
      bimg.getGraphics().drawString("Version: " + ArianeSettings.VERSION_ID, 80, 220);

      return new ImageIcon(bimg);
   }

   public void showProgress( final String message, final int percent ) {
      _progressBar.showStatus(message);
      _progressBar.showProgress(percent);
   }

   private final class ProgressBar extends JPanel {

      private final JLabel _messageText = new JLabel();
      private final JProgressBar _progressInidicator = new JProgressBar();

      ProgressBar() {
         _progressInidicator.setMinimum(0);
         _progressInidicator.setMaximum(100);
         _progressInidicator.setMinimumSize(new Dimension(100, 20));
         _progressInidicator.setSize(new Dimension(100, 20));

         _messageText.setMinimumSize(new Dimension(300, 20));
         _messageText.setSize(new Dimension(300, 20));
         _messageText.setFont(_messageText.getFont().deriveFont(Font.BOLD, 9f));
         _messageText.setForeground(Color.black);

         setLayout(new BorderLayout());
         setBorder(new EtchedBorder(EtchedBorder.LOWERED));
         add(_messageText, BorderLayout.CENTER);
         add(_progressInidicator, BorderLayout.EAST);
      }

      public void showStatus( final String message ) {
         _messageText.setText(message);
         paintImmediately(getBounds());
      }

      public void showProgress( final int percent ) {
         _progressInidicator.setValue(percent);
      }
   }
}
