/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.swingext.GDialog;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.model.UserPreferences.NamedColor;

/**
 * The class <code>UserPreferencesChooser</code> allows the user to set the preferences.
 */
public final class UserPreferencesChooser extends GDialog {

   private boolean _isValidated;
   private String _language;
   private int _defaultFontSize;
   private String _temporaryFileExtension = ".tmp";
   private NamedColor[] _colors;
   private int _historySize;

   private final JTabbedPane _prefsTabbedPane = new JTabbedPane();
   private static int _lastestSelectedTab;

   private final JTextField _visuCommand = new JTextField(10);
   private final JTextField _statusCommand = new JTextField(10);
   private final JTextField _browserPath = new JTextField(10);
   private final JTextField _editorPath = new JTextField(10);
   private final JTextField _dataFileDirectory = new JTextField(20);
   private final JTextField _binaryDirectory = new JTextField(20);
   private final JTextField _temporaryDirectory = new JTextField(20);

   private final JCheckBox _toolTipState = new JCheckBox();

   private JSpinner _undoSizeSpinner;

   public UserPreferencesChooser() {
      super(Ariane.getFrame(), ArianeSettings.getString("PreferenceChooser"), true);
      setResizable(false);
      displayContent();
      pack();
      setLocationRelativeTo(getOwner());
      addWindowListener(new WindowAdapter() {
         @Override
         public void windowClosing( WindowEvent event ) {
            cancelAction();
         }
      });
   }

   private Box createGeneralPreferencesTab() {
      Box panel = Box.createVerticalBox();

      // Language
      JPanel panel1 = new JPanel();
      panel.add(panel1);
      panel1.setLayout(new FlowLayout(FlowLayout.LEFT));
      panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("PreferenceChooser.general.language")));
      panel1.add(new JLabel(ArianeSettings.getString("PreferenceChooser.general.language.label")));
      String[] languages = {"Français", "English"};
      JComboBox<String> list = new JComboBox<>(languages);
      list.setSelectedItem(_language);
      list.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            @SuppressWarnings("unchecked")
            JComboBox<String> comboBox = (JComboBox<String>) event.getSource();
            _language = (String) comboBox.getSelectedItem();
         }
      });
      panel1.add(list);

      // tooltips
      JPanel panel2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
      panel.add(panel2);
      panel2.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("PreferenceChooser.general.tooltip")));
      panel2.add(_toolTipState);
      panel2.add(new JLabel(ArianeSettings.getString("PreferenceChooser.general.tooltip.text")));

      // History size
      JPanel panel3 = new JPanel(new FlowLayout(FlowLayout.LEFT));
      panel.add(panel3);
      panel3.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED), ArianeSettings.getString("PreferenceChooser.undoHistoryTitle")));
      SpinnerModel spinnerModel;
      spinnerModel = new SpinnerNumberModel(_historySize, 10, 500, 1);
      _undoSizeSpinner = new JSpinner(spinnerModel);
      panel3.add(new JLabel(ArianeSettings.getString("PreferenceChooser.undoHistoryText")));
      panel3.add(_undoSizeSpinner);

      panel.add(softwareSelector("PreferenceChooser.general.browser", "PreferenceChooser.general.browser.text", _browserPath));
      panel.add(softwareSelector("PreferenceChooser.general.editor", "PreferenceChooser.general.editor.text", _editorPath));

      panel.add(Box.createGlue());
      return panel;
   }

   private Component softwareSelector( String title, String labelText, final JTextField component ) {
      JPanel panel4 = new JPanel(new FlowLayout(FlowLayout.LEFT));
      panel4.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString(title)));
      panel4.add(new JLabel(ArianeSettings.getString(labelText)));
      panel4.add(component);
      JButton browser1 = new JButton(ArianeSettings.getString("Ariane.browse"));
      panel4.add(browser1);
      browser1.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            selectSoftware(component);
         }
      });
      return panel4;
   }

   private void selectSoftware( JTextField textfield ) {
      final JFileChooser fileChooser = new JFileChooser();
      fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
      fileChooser.setDialogTitle(ArianeSettings.getString("PreferenceChooser.general.selectProgram"));
      fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
      fileChooser.setCurrentDirectory(new File("."));
      if (fileChooser.showOpenDialog(UserPreferencesChooser.this) == JFileChooser.APPROVE_OPTION) {
         textfield.setText(fileChooser.getSelectedFile().getAbsolutePath());
      }
   }

   private Box createRuntimePreferencesTab() {
      Box panel = Box.createVerticalBox();

      panel.add(softwareSelector("PreferenceChooser.runtime.visuCommand", "PreferenceChooser.runtime.CommandName", _visuCommand));
      panel.add(softwareSelector("PreferenceChooser.runtime.statusCommand", "PreferenceChooser.runtime.CommandName", _statusCommand));
      return panel;
   }

   private Box createGraphPreferencesTab() {
      Box panel = Box.createVerticalBox();

      // List of known languages.
      JPanel panel1 = new JPanel();
      panel.add(panel1);
      panel1.setLayout(new FlowLayout(FlowLayout.LEFT));
      panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("PreferenceChooser.graph.font")));

      panel1.add(new JLabel(ArianeSettings.getString("PreferenceChooser.graph.font.size")));
      String[] fontSize = {"8", "9", "10", "11", "12"};
      JComboBox<String> list = new JComboBox<>(fontSize);
      list.setSelectedItem(Integer.toString(_defaultFontSize));
      list.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            @SuppressWarnings("unchecked")
            JComboBox<String> comboBox = (JComboBox<String>) event.getSource();
            _defaultFontSize = Integer.parseInt((String) comboBox.getSelectedItem());
         }
      });
      panel1.add(list);
      panel1.add(new JLabel(ArianeSettings.getString("PreferenceChooser.graph.font.notice")));

      createColorSquare(panel, 5, _colors.length);

      return panel;
   }

   private Box createColorPreferencesTab() {
      Box panel = Box.createVerticalBox();
      createColorSquare(panel, 0, 5);
      return panel;
   }

   private void createColorSquare( Box panel, int start, int end ) {
      for (int i = start; i < end; i++) {
         JPanel panel1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
         panel.add(panel1);
         panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), _colors[i].getName()));
         panel1.add(new BindingColor(ArianeSettings.getString("PreferenceChooser.backgroundColor"), _colors[i]));
      }
   }

   private Box createPathPreferencesTab() {
      Box panel = Box.createVerticalBox();
      JPanel panel1 = directorySelector("PreferenceChooser.path.temporaryDirectory", _temporaryDirectory);
      panel.add(panel1);

      JPanel panel2 = temporaryFileExtension(panel);
      panel.add(panel2);

      panel.add(Box.createGlue());

      JPanel panel3 = directorySelector("PreferenceChooser.path.fileDirectory", _dataFileDirectory);
      panel.add(panel3);
      JPanel panel4 = directorySelector("PreferenceChooser.path.binaryDirectory", _binaryDirectory);
      panel.add(panel4);

      return panel;
   }

   private JPanel directorySelector( String title, final JTextField textfield ) {
      JPanel panel3 = new JPanel();
      panel3.setLayout(new FlowLayout(FlowLayout.LEFT));
      panel3.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString(title)));
      JButton button3 = new JButton(ArianeSettings.getString("Ariane.browse"));
      button3.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            selectDirectory(textfield);
         }
      });
      panel3.add(textfield);
      panel3.add(button3);
      return panel3;
   }

   private void selectDirectory( final JTextField textfield ) {
      final JFileChooser fileChooser = new JFileChooser();
      fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
      fileChooser.setDialogTitle(ArianeSettings.getString("FileChooser.titleSelectDir"));
      fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
      String directory = textfield.getText();
      if (directory != null) {
         fileChooser.setCurrentDirectory(new File(directory));
      }
      if (fileChooser.showOpenDialog(UserPreferencesChooser.this) == JFileChooser.APPROVE_OPTION) {
         String nDirectory = fileChooser.getSelectedFile().getAbsolutePath();
         textfield.setText(nDirectory);
      }
   }

   private JPanel temporaryFileExtension( Box panel ) {
      JPanel panel2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
      panel2.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("PreferenceChooser.runtime.TemporaryFileExtension")));
      JTextField textfield2 = new JTextField(20);
      panel2.add(new JLabel(ArianeSettings.getString("PreferenceChooser.runtime.fileExtension")));

      textfield2.setText(_temporaryFileExtension);
      textfield2.addFocusListener(new FocusAdapter() {
         @Override
         public void focusLost( FocusEvent event ) {
            _temporaryFileExtension = ((JTextField) event.getSource()).getText();
         }
      });
      panel2.add(textfield2);
      panel.add(Box.createGlue());

      JPanel panel21 = new JPanel(new FlowLayout(FlowLayout.LEFT));
      panel.add(panel21);
      panel21.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("PreferenceChooser.environment.title")));
      // -> Remove temporary files
      panel21.add(new JLabel(ArianeSettings.getString("PreferenceChooser.environment.cleaning")));
      JButton button2 = new JButton(ArianeSettings.getString("PreferenceChooser.environment.erasing"));
      button2.addActionListener(new ActionListener() { // Erases all temporary files.
         @Override
         public void actionPerformed( ActionEvent event ) {
            int numberOfFiles = UserPreferences.deleteTemporaryFiles();
            if (numberOfFiles > 0) {
               JOptionPane.showMessageDialog(UserPreferencesChooser.this,
                   numberOfFiles + ArianeSettings.getString("PreferenceChooser.message1"),
                   ArianeSettings.getString("notice"),
                   JOptionPane.INFORMATION_MESSAGE);
            }
         }
      });
      panel21.add(button2);
      return panel2;
   }

   private void displayContent() {
      initializePreferences();

      _prefsTabbedPane.addTab(ArianeSettings.getString("PreferenceChooser.general.title"), createGeneralPreferencesTab());
      _prefsTabbedPane.addTab(ArianeSettings.getString("PreferenceChooser.runtime.title"), createRuntimePreferencesTab());
      _prefsTabbedPane.addTab(ArianeSettings.getString("PreferenceChooser.graph.title"), createGraphPreferencesTab());
      _prefsTabbedPane.addTab(ArianeSettings.getString("PreferenceChooser.item.title"), createColorPreferencesTab());
      _prefsTabbedPane.addTab(ArianeSettings.getString("PreferenceChooser.paths"), createPathPreferencesTab());
      _prefsTabbedPane.setSelectedIndex(_lastestSelectedTab); // persistence

      // Buttons panel
      final JPanel buttonPanel = new JPanel();
      JButton applyButton = new JButton(ArianeSettings.getString("PreferenceChooser.applyText"), ArianeSettings.getIcon("gtk-apply.png"));
      applyButton.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            buttonPanel.requestFocusInWindow(); // Force focus out in the textfields...
            SwingUtilities.invokeLater(new Runnable() {
               // Tip: handle this dialog events before applying user preferences.
               @Override
               public void run() {
                  applyAction();
               }
            });
         }
      });

      // cancel button
      JButton cancelButton = new JButton();
      cancelButton.setAction(new AbstractAction(ArianeSettings.getString("TreeEditor.cancel"), ArianeSettings.getIcon("gtk-cancel.png")) {
         @Override
         public void actionPerformed( ActionEvent event ) {
            cancelAction();
         }
      });

      JButton helpButton = new JButton();
      helpButton.setAction(new AbstractAction(null, ArianeSettings.getIcon("help.png")) {
         @Override
         public void actionPerformed( ActionEvent event ) {
            setVisible(false);
            Ariane.getPresenter().displayDocumentationPage("Ariane", "manual.html", "#tab" + _prefsTabbedPane.getSelectedIndex());
         }
      });
      buttonPanel.add(helpButton);
      buttonPanel.add(Box.createHorizontalStrut(80));
      buttonPanel.add(applyButton);
      buttonPanel.add(cancelButton);
      setDefaultButton(applyButton);
      setCancelButton(cancelButton);

      getContentPane().setLayout(new BorderLayout());
      getContentPane().add(buttonPanel, BorderLayout.SOUTH);
      getContentPane().add(_prefsTabbedPane, BorderLayout.CENTER);
      pack();
   }

   private void initializePreferences() {
      _language = UserPreferences.getLanguage();
      _toolTipState.setSelected(UserPreferences.isToolTipEnabled());
      _historySize = UserPreferences.getHistorySize();
      _browserPath.setText(UserPreferences.getWebBrowserPath());
      _editorPath.setText(UserPreferences.getTextEditorPath());
      _visuCommand.setText(UserPreferences.getCommandVisu());
      _statusCommand.setText(UserPreferences.getCommandStatus());
      _colors = UserPreferences.getConfigurableColors();
      _defaultFontSize = UserPreferences.getDefaultFontSize();
      _temporaryDirectory.setText(UserPreferences.getTemporaryDirectoryBase());
      _dataFileDirectory.setText(UserPreferences.getPreferredFileDirectory());
      _temporaryFileExtension = UserPreferences.getTemporaryFileExtension();
      _binaryDirectory.setText(UserPreferences.getPreferredBinaryDirectory());
   }

   private void cancelAction() {
      _lastestSelectedTab = _prefsTabbedPane.getSelectedIndex();
      setVisible(false);
      dispose();
   }

   private void applyAction() {
      changePreferences();
      _lastestSelectedTab = _prefsTabbedPane.getSelectedIndex();
      _isValidated = true;
      setVisible(false);
      dispose();
   }

   public boolean isValidated() {
      return _isValidated;
   }

   private void changePreferences() {
      UserPreferences.changeLanguage(_language);
      UserPreferences.setHistorySize((Integer) _undoSizeSpinner.getValue());
      UserPreferences.setToolTipState(_toolTipState.isSelected());
      UserPreferences.setWebBrowserPath(_browserPath.getText().trim());
      UserPreferences.setTextEditorPath(_editorPath.getText().trim());

      UserPreferences.setCommandVisu(_visuCommand.getText().trim());
      UserPreferences.setCommandStatus(_statusCommand.getText().trim());

      UserPreferences.setConfigurableColors(_colors);
      UserPreferences.setDefaultFontSize(_defaultFontSize);
      try {
         UserPreferences.setTemporaryDirectoryBase(_temporaryDirectory.getText());
      } catch (IOException e) {
         Ariane.getPresenter().printErrorMessage(e.getMessage());
      }

      UserPreferences.setPreferedFileDirectory(_dataFileDirectory.getText().trim());
      UserPreferences.setBinaryDirectory(_binaryDirectory.getText().trim());
      UserPreferences.setTemporaryFileExtension(_temporaryFileExtension);
   }

   /**
    * The class <code>BindingColor</code> is an inner class that represents a pair (item, color). It provides a pane of controls designed to allow a user to choose a color for a given item.
    */
   private final class BindingColor extends JPanel {

      private final NamedColor _color;
      private final String _legend;
      private final JPanel _colorButton;

      BindingColor( final String legend, final NamedColor numColor ) {
         _legend = legend;
         _color = numColor;
         _colorButton = new JPanel();
         _colorButton.setPreferredSize(new Dimension(30, 30));
         _colorButton.setBorder(BorderFactory.createEtchedBorder());
         _colorButton.setBackground(_color.getColor());
         _colorButton.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked( final MouseEvent event ) {
               selectColor(event.getComponent());
            }
         });
         setLayout(new BorderLayout());
         JLabel label = new JLabel(_legend + "   ");
         label.setFont(new Font("Dialog", Font.PLAIN, 12));
         add(label, BorderLayout.CENTER);
         add(_colorButton, BorderLayout.EAST);
      }

      private void selectColor( Component item ) {
         final Color color = JColorChooser.showDialog(item, _legend, _color.getColor());
         if (color != null) {
            _color.setColor(color);
            _colorButton.setBackground(color);
            _colorButton.repaint();
         }
      }
   }
}
