/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.explorer;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.model.graph.item.OperatorItem;
import fr.greyc.pantheon.ariane.model.graph.item.etc.ItemSignatureParser;
import fr.greyc.pantheon.ariane.model.tree.MutableTreeItem;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.swingext.GDialog;
import fr.greyc.pantheon.swingext.HTMLDocumentEditor;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.lang.reflect.MalformedParameterizedTypeException;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.tree.DefaultTreeModel;

/**
 * The class <code>OperatorEditor</code> is a dialog box that
 * allows user to edit operator properties.
 */
public final class OperatorEditor extends GDialog {

   private TreeItem _currentOperator;
   private final MutableTreeItem _currentNode;
   private final DefaultTreeModel _treeModel;
   private JTextField _textfield1;
   private JTextField _textfield2;
   private JTextField _textfield2bis;
   private JTextField _docURLField;
   private JTextField _signature;
   private JSpinner _textFieldiP;
   private JSpinner _textFieldiI;
   private JSpinner _textFieldiO;
   private static String _currentDirectory = UserPreferences.getPreferredBinaryDirectory();

   public OperatorEditor( JDialog parent, MutableTreeItem currentNode, DefaultTreeModel treeModel ) {
      super(parent, ArianeSettings.getString("OperatorEditor.title"), true);
      _currentNode = currentNode;
      _treeModel = treeModel;
      _currentOperator = (TreeItem) _currentNode.getUserObject();

      addWindowListener(new WindowAdapter() {
         @Override
         public void windowClosing( WindowEvent event ) {
            cancelAction();
         }
      });
      displayContent();
      setLocationRelativeTo(parent);
   }

   private void displayContent() {
      JPanel textPanel = new JPanel(new GridBagLayout());
      textPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("OperatorEditor.borderTitle")));

      GridBagConstraints cst = new GridBagConstraints();
      cst.fill = GridBagConstraints.BOTH;
      cst.anchor = GridBagConstraints.WEST;
      textPanel.add(new JLabel(ArianeSettings.getString("OperatorEditor.name")), cst);
      _textfield1 = new JTextField(30);
      _textfield1.setText(_currentOperator.getName());

      cst.gridwidth = GridBagConstraints.REMAINDER;
      textPanel.add(_textfield1, cst);

      cst.gridwidth = 1;
      textPanel.add(new JLabel(ArianeSettings.getString("OperatorEditor.binary")), cst);
      _textfield2 = new JTextField(30);
      String binary = UserPreferences.getAbsoluteBinary(OperatorItem.getCommandPart(_currentOperator.getBinaryPath()));
      if (binary != null && binary.length() > 0) {
         _textfield2.setText(binary);
      }
      textPanel.add(_textfield2, cst);
      JButton button1 = createButtonLoad(_textfield2, "");
      cst.gridwidth = GridBagConstraints.REMAINDER;
      textPanel.add(button1, cst);

      cst.gridwidth = 1;
      textPanel.add(new JLabel(ArianeSettings.getString("OperatorEditor.synopsis")), cst);
      _textfield2bis = new JTextField(35);
      String format = OperatorItem.getFormatPart(_currentOperator.getBinaryPath());
      _textfield2bis.setText(format);
      cst.gridwidth = GridBagConstraints.REMAINDER;
      textPanel.add(_textfield2bis, cst);

      cst.gridwidth = GridBagConstraints.REMAINDER;
      textPanel.add(new JLabel("<html><i>" + ArianeSettings.getString("OperatorEditor.synopsisDescription") + "</i><html>"), cst);

      cst.gridwidth = 1;
      textPanel.add(new JLabel(ArianeSettings.getString("OperatorEditor.manual")), cst);
      _docURLField = new JTextField(35);
      _docURLField.setText(_currentOperator.getDocumentationURL());
      textPanel.add(_docURLField, cst);
      JButton button2 = createButtonLoad(_docURLField, "file://");

      JButton button3 = new JButton(ArianeSettings.getString("OperatorEditor.edit"));
      button3.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            HTMLDocumentEditor editor = new HTMLDocumentEditor(OperatorEditor.this);
            String prefix = "file://";
            if (_docURLField.getText().length() > prefix.length()) {
               editor.setSelectedFile(new File(_docURLField.getText().substring(prefix.length())));
            }
            if (editor.edit() == HTMLDocumentEditor.OK_OPTION && editor.getSelectedFile() != null) {
               File file = editor.getSelectedFile();
               _docURLField.setText(prefix + file.getAbsolutePath());
               _docURLField.setCaretPosition(0);
            }
         }
      });
      textPanel.add(button2, cst);
      cst.gridwidth = GridBagConstraints.REMAINDER;
      textPanel.add(button3, cst);

      cst.gridwidth = 1;
      cst.fill = GridBagConstraints.NONE;

      textPanel.add(new JLabel(ArianeSettings.getString("OperatorEditor.parameters")), cst);
      SpinnerModel spModel = new SpinnerNumberModel(_currentOperator.getNumberOfParameterSlots(), 0, 10, 1);
      _textFieldiP = new JSpinner(spModel);
      cst.gridwidth = GridBagConstraints.REMAINDER;
      textPanel.add(_textFieldiP, cst);

      cst.gridwidth = 1;
      textPanel.add(new JLabel(ArianeSettings.getString("OperatorEditor.signature")), cst);
      _signature = new JTextField(30);
      String defaultValues = _currentOperator.getSignature();
      _signature.setText(defaultValues);
      cst.gridwidth = GridBagConstraints.REMAINDER;
      textPanel.add(_signature, cst);
      textPanel.add(new JLabel("<html><i>" + ArianeSettings.getString("OperatorEditor.signatureExample") + "</i></html>"), cst);

      cst.gridwidth = 1;
      textPanel.add(new JLabel(ArianeSettings.getString("OperatorEditor.inputs")), cst);
      spModel = new SpinnerNumberModel(_currentOperator.getNumberOfInputDataSlots(), 0, 10, 1);
      _textFieldiI = new JSpinner(spModel);
      cst.gridwidth = GridBagConstraints.REMAINDER;
      textPanel.add(_textFieldiI, cst);

      cst.gridwidth = 1;
      textPanel.add(new JLabel(ArianeSettings.getString("OperatorEditor.outputs")), cst);
      spModel = new SpinnerNumberModel(_currentOperator.getNumberOfOutputDataSlots(), 0, 10, 1);
      _textFieldiO = new JSpinner(spModel);
      cst.gridwidth = GridBagConstraints.REMAINDER;
      textPanel.add(_textFieldiO, cst);

      // cancel button
      JButton cancelButton = new JButton(new AbstractAction(ArianeSettings.getString("OperatorEditor.cancel"), ArianeSettings.getIcon("gtk-cancel.png")) {
         @Override
         public void actionPerformed( ActionEvent event ) {
            cancelAction();
         }
      });

      JButton closeButton = new JButton(ArianeSettings.getString("OperatorEditor.validate"), ArianeSettings.getIcon("gtk-apply.png"));
      JPanel buttonPanel = new JPanel();
      closeButton.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            applyAction();
         }
      });
      buttonPanel.add(closeButton);
      buttonPanel.add(cancelButton);
      setDefaultButton(closeButton);
      setCancelButton(cancelButton);

      getContentPane().setLayout(new BorderLayout());
      getContentPane().add(textPanel, BorderLayout.CENTER);
      getContentPane().add(buttonPanel, BorderLayout.SOUTH);
      pack();
   }

   private JButton createButtonLoad( final JTextField textfield, final String prefix ) {
      JButton button = new JButton(ArianeSettings.getString("OperatorEditor.load"));
      button.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            JFileChooser fileChooser = new JFileChooser(_currentDirectory);
            fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
            fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
            int prefixLength = prefix.length();
            if (textfield.getText().length() > prefixLength) {
               fileChooser.setSelectedFile(new File(textfield.getText().substring(prefixLength)));
            }
            if (fileChooser.showOpenDialog(OperatorEditor.this) == JFileChooser.APPROVE_OPTION) {
               textfield.setText(prefix + fileChooser.getSelectedFile().getAbsolutePath());
               textfield.setCaretPosition(0);
               _currentDirectory = fileChooser.getSelectedFile().getParentFile().getAbsolutePath();
            }
         }
      });
      return button;
   }

   private void cancelAction() {
      setVisible(false);
      dispose();
   }

   private void applyAction() {
      try {
         _currentOperator.setName(_textfield1.getText());
         _currentOperator.setBinaryPath(_textfield2.getText() + OperatorItem.SEPARATOR + _textfield2bis.getText());
         _currentOperator.setDocumentationURL(_docURLField.getText());
         _currentOperator.setNumberOfParameterSlots((Integer) _textFieldiP.getValue());
         _currentOperator.setNumberOfInputDataSlots((Integer) _textFieldiI.getValue());
         _currentOperator.setNumberOfOutputDataSlots((Integer) _textFieldiO.getValue());
         _treeModel.nodeChanged(_currentNode);
         ItemSignatureParser.parse(_signature.getText(), _currentOperator.getNumberOfParameterSlots());
         _currentOperator.setSignature(_signature.getText());
      } catch (NumberFormatException e) {
         Ariane.getPresenter().printErrorMessage(e.getLocalizedMessage());
      } catch (MalformedParameterizedTypeException e) {
         Ariane.getPresenter().printErrorMessage(ArianeSettings.getString("OperatorEditor.error1"));
      }
      setVisible(false);
      _currentOperator = null;
      dispose();
   }
}
