/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.explorer;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.swing.JDialog;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.tree.MutableTreeItem;
import fr.greyc.pantheon.ariane.model.tree.OperatorTreeItem;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.model.xml.XMLFileOutputStream;
import fr.greyc.pantheon.swingext.GDialog;

/**
 * The class <code>OperatorTreeEditor</code> defines a dialog box
 * where user can modify the personal operators tree.
 */
public final class OperatorTreeEditor extends TreeEditor {

   public OperatorTreeEditor( MutableTreeItem rootNode ) {
      super(ArianeSettings.getString("OperatorTreeEditor.title"), rootNode);
   }

   @Override
   protected JDialog getItemEditor( GDialog dialog, MutableTreeItem currentNode, DefaultTreeModel treeModel ) {
      return new OperatorEditor(dialog, currentNode, treeModel);
   }

   @Override
   protected TreeItem createItem( String categoryName ) {
      TreeItem treeItem = new OperatorTreeItem(TreeItem.Item.OPERATOR, "operator", "", 0, 0, 0, "");
      treeItem.setCategory(categoryName);
      return treeItem;
   }

   @Override
   protected boolean isNewEnabled( TreePath path ) {
      return path != null && path.getPathCount() < 3;
   }

   @Override
   protected void openPopupMenuCategory( final JTree tree, int x, int y ) {
      JPopupMenu popupMenu = new JPopupMenu();

      JMenuItem item = new JMenuItem(ArianeSettings.getString("OperatorTreeEditor.addOperator"));
      item.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            addNewItemBelowCurrentSelection();
         }
      });
      popupMenu.add(item);
      item = new JMenuItem(ArianeSettings.getString("TreeEditor.editCategory"));
      item.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            editCurrentItem();
         }
      });
      popupMenu.add(item);
      item = new JMenuItem(ArianeSettings.getString("TreeEditor.deleteCategory"));
      item.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            deleteSelectedItem();
         }
      });
      popupMenu.add(item);
      popupMenu.show(tree, x, y);
   }

   @Override
   protected void openPopupMenuItem( final JTree tree, int x, int y ) {
      JPopupMenu popupMenu = new JPopupMenu();
      JMenuItem item;
      item = new JMenuItem(ArianeSettings.getString("TreeEditor.modify"));
      item.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            editCurrentItem();
         }
      });
      popupMenu.add(item);
      item = new JMenuItem(ArianeSettings.getString("TreeEditor.delete"));
      item.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            deleteSelectedItem();
         }
      });
      popupMenu.add(item);
      popupMenu.show(tree, x, y);
   }

   @Override
   protected boolean storeInPreferenceFile( MutableTreeItem node ) {
      Ariane.getPresenter().getTree().setUserOperatorTree(node);

      File file = new File(ArianeSettings.getPersonalOperatorFilename());
      File directory = file.getParentFile();
      try {
         directory.mkdirs();
         XMLFileOutputStream out = new XMLFileOutputStream(file, null);
         out.write(convertToXMLTree());
         out.close();
      } catch (IOException e) {
         Ariane.getPresenter().printErrorMessage(ArianeSettings.getString("PreferenceChooser.error.saved") + " " + ArianeSettings.getPreferencesFilename() + " " + e.getLocalizedMessage());
         return false;
      }
      return true;
   }

   private XMLElement convertToXMLTree() throws IOException {
      List<TreeItem> operatorList = Ariane.getPresenter().getTree().getUserOperatorTreeAsList();
      if (!operatorList.isEmpty()) {
         XMLElement element = new XMLElement("OperatorLibrary");
         for (TreeItem item : operatorList) {
            element.addChildren(item.convertToXMLTree());
         }
         return element;
      }
      return null;
   }
}
