/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph.item;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.model.graph.item.ControlItem;
import fr.greyc.pantheon.ariane.model.graph.slot.InputParameterSlot;
import fr.greyc.pantheon.swingext.FadingEffects;
import fr.greyc.pantheon.swingext.GDialog;

public abstract class ControlItemView extends GraphItemView {

   private boolean _validatedPrototype;

   ///// 
   ///// P R O P E R T I E S
   /////
   public final void askProperties( String name, int defaultInputImageNumber, int defaultParameterNumber, int defaultOutputImageNumber ) throws InterruptedException {
      final GDialog dialog = new GDialog((JFrame) null, getModel().getTagName(), true);
      dialog.setAlwaysOnTop(true);
      dialog.setResizable(false);
      _validatedPrototype = false;

      String nickName;
      if (name == null) {
         nickName = getName();
      } else {
         nickName = name;
      }
      final JTextField textFieldname = new JTextField(nickName, 10);
      SpinnerModel spModel;
      spModel = new SpinnerNumberModel(defaultParameterNumber, 0, 10, 1);
      final JSpinner textFieldiP = new JSpinner(spModel);
      spModel = new SpinnerNumberModel(defaultInputImageNumber, 0, 10, 1);
      final JSpinner textFieldiIm = new JSpinner(spModel);
      spModel = new SpinnerNumberModel(defaultOutputImageNumber, 0, 10, 1);
      final JSpinner textFieldoIm = new JSpinner(spModel);

      JPanel jPanel = new JPanel(new GridBagLayout());
      jPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("MacroOperator.borderTitle")));

      GridBagConstraints constraint = new GridBagConstraints();
      constraint.fill = GridBagConstraints.NONE;
      constraint.anchor = GridBagConstraints.WEST;
      constraint.ipadx = 8;
      constraint.ipady = 8;
      constraint.gridy = 0;

      if (name != null) {
         constraint.gridy++;
         constraint.gridx = 0;
         jPanel.add(new JLabel(ArianeSettings.getString("MacroOperator.name")), constraint);
         constraint.gridx = 1;
         jPanel.add(textFieldname, constraint);
      }
      if (defaultParameterNumber > 0) {
         constraint.gridy++;
         constraint.gridx = 0;
         jPanel.add(new JLabel(ArianeSettings.getString("MacroOperator.parameternumber")), constraint);
         constraint.gridx = 1;
         jPanel.add(textFieldiP, constraint);
      }
      if (defaultInputImageNumber > 0) {
         constraint.gridy++;
         constraint.gridx = 0;
         jPanel.add(new JLabel(ArianeSettings.getString("MacroOperator.inputnumber")), constraint);
         constraint.gridx = 1;
         jPanel.add(textFieldiIm, constraint);
      }
      if (defaultOutputImageNumber > 0) {
         constraint.gridy++;
         constraint.gridx = 0;
         jPanel.add(new JLabel(ArianeSettings.getString("MacroOperator.outputnumber")), constraint);
         constraint.gridx = 1;
         jPanel.add(textFieldoIm, constraint);
      }

      JButton applyButton = new JButton(ArianeSettings.getString("PropertyViewer.ok"), ArianeSettings.getIcon("gtk-yes.png"));

      applyButton.addActionListener(new ActionListener() {
         @Override
         public final void actionPerformed( ActionEvent event ) {
            int ii = (Integer) textFieldiIm.getValue();
            int ip = (Integer) textFieldiP.getValue();
            int io = (Integer) textFieldoIm.getValue();
            String newName = textFieldname.getText();
            ((ControlItem) getModel()).setPrototype(newName, ii, ip, io);
            _validatedPrototype = true;
            dialog.setVisible(false);
         }
      });

      JButton cancelButton = new JButton();
      cancelButton.setAction(new AbstractAction(ArianeSettings.getString("SubroutineEditor.cancel"),
          ArianeSettings.getIcon("gtk-cancel.png")) {
         @Override
         public final void actionPerformed( ActionEvent event ) {
            _validatedPrototype = false;
            dialog.setVisible(false);
         }
      });

      dialog.getContentPane().setLayout(new BorderLayout(5, 5));
      dialog.getContentPane().add(jPanel, BorderLayout.CENTER);
      JPanel panel = new JPanel();
      panel.add(applyButton);
      panel.add(cancelButton);
      dialog.setDefaultButton(applyButton);
      dialog.setCancelButton(cancelButton);
      dialog.getContentPane().add(panel, BorderLayout.SOUTH);
      dialog.pack();
      dialog.setLocationRelativeTo(Ariane.getFrame());
      FadingEffects.fadeIn(Ariane.getFrame());
      dialog.setVisible(true);
      FadingEffects.fadeOut(Ariane.getFrame());

      if (!_validatedPrototype) {
         throw new InterruptedException();
      }
   }

   ///// 
   ///// P R I N T I N G
   /////
   @Override
   public final void printComponent( Graphics gc ) {
      super.printComponent(gc);
      gc.setColor(Color.black);
      gc.setFont(UserPreferences.getDefaultFont());
      FontMetrics fm = gc.getFontMetrics();
      int row = fm.getHeight();
      // Name
      super.centerText(getModel().getNickName(), gc, row, false);

      // Print each parameters.
      // Print the value only of if the parameter is not connected.
      for (int i = 0; i < getModel().getInputParameterSlots().size(); i++) {
         InputParameterSlot param = getModel().getInputParameterSlots().get(i);
         row += fm.getHeight() + 1;
         if (param.isConnected()) {
            super.centerText("$" + i + ":<linked>", gc, row, true);
         } else {
            super.centerText("$" + i + ":" + param.getValue(), gc, row, true);
         }
      }
      row += fm.getHeight() + 2;
      if (getFormula() != null) {
         super.centerText(getFormula(), gc, row, true);
      }
   }
}
