/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph.item;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;

import javax.swing.JPopupMenu;

import fr.greyc.pantheon.ariane.model.graph.slot.InputParameterSlot;
import fr.greyc.pantheon.ariane.view.graph.item.etc.OperatorColor;

public final class OperatorItemView extends GraphItemView {

   @Override
   public void printComponent( Graphics gc ) {
      super.printComponent(gc);
      FontMetrics fontMetrics = gc.getFontMetrics();
      int row = fontMetrics.getHeight();

      // The name of the operator.
      centerText(getModel().getNickName(), gc, row, false);
      // Print each parameters.
      // Print the value only of if the parameter is not connected.
      for (int i = 0; i < getModel().getInputParameterSlots().size(); i++) {
         InputParameterSlot param = getModel().getInputParameterSlots().get(i);
         row += fontMetrics.getHeight() + 1;
         if (param.isConnected()) {
            centerText("$" + i + ":<linked>", gc, row, true);
         } else {
            centerText("$" + i + ":" + param.getValue(), gc, row, true);
         }
      }
   }

   ///// 
   ///// C O L O R
   /////
   @Override
   public Color getBackgroundColor() {
      return OperatorColor.getColor();
   }

   ///// 
   ///// M E N U 
   /////
   @Override
   protected JPopupMenu initPopup() {
      JPopupMenu popupMenu = new JPopupMenu();

      popupMenu.add(createExecuteMenuItem());
      popupMenu.add(createResetMenuItem());

      popupMenu.addSeparator();

      popupMenu.add(createCopyMenuItem());
      popupMenu.add(createCutMenuItem());
      popupMenu.add(createDeleteMenuItem());

      return popupMenu;
   }
}
