/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph.item;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.model.graph.item.DataFile;
import fr.greyc.pantheon.ariane.presenter.command.ChangeDatafileEdit;
import fr.greyc.pantheon.ariane.view.PropertyPanel;

public final class SaverItemView extends DataFileView {

   private static StaticSaveFileChooser _fileChooser = new StaticSaveFileChooser();
   private final JButton _jbuttonSelectFile = new JButton();
   private final JTextField _binaryEditor = new JTextField("");

   private static String _currentSavingFolder;

   public SaverItemView() {
      super("saver");
   }

   /**
    * @return a component appropriate to edit and possibly change the name of the item.
    */
   @Override
   public JComponent getBinaryEditor() {
      return null;
   }

   @Override
   public void changeBinaryName( JTextField textfield1, String binary, boolean addInHistory ) {
      JTextField textfield2 = _binaryEditor;
      JButton button = _jbuttonSelectFile;

      getModel().resetAll();
      final String oldBinary = getModel().getBinary();
      getModel().setBinary(binary);

      if (getName().equals(getModel().getNickName())
          || oldBinary != null && DataFile.getFileNameWithoutExtension(oldBinary).equals(getModel().getNickName())) {
         if (binary == null) {
            getModel().setNickName(null);
         } else {
            getModel().setNickName(DataFile.getFileNameWithoutExtension(binary));
         }
      }
      textfield1.setText(getModel().getNickName());
      textfield2.setText(getModel().getBinary());
      if (button != null) {
         if (binary == null) {
            resetButtonText(button);
         } else {
            button.setText(DataFile.getFileNameWithoutExtension(binary));
         }
      }

      getModel().updateName();
      getGraph().setModified(true);
      if (addInHistory) {
         getGraph().validateOneAction(new ChangeDatafileEdit((DataFile) this.getModel(), textfield1, binary, oldBinary));
      }
   }

   @Override
   protected void onMouseDoubleClicked( MouseEvent event ) {
      selectDataFile();
   }

   private void selectDataFile() {
      if (_currentSavingFolder == null) {
         _currentSavingFolder = UserPreferences.getCurrentDirectory();
      }
      _fileChooser.setCurrentDirectory(new File(_currentSavingFolder));
      final String currentFile = getModel().getBinary();
      if (currentFile != null && !currentFile.trim().isEmpty()) {
         _fileChooser.setSelectedFile(new File(currentFile));
      }
      if (_fileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
         final String path = _fileChooser.getSelectedFile().getPath();
         ((DataFile) getModel()).setCurrentFilename(null);
         _currentSavingFolder = _fileChooser.getSelectedFile().getParent();
         changeBinaryName((JTextField) getNameEditor(), path, true);
      }
   }

   @Override
   protected void resetButtonText( JButton button ) {
      button.setText(ArianeSettings.getString("PropertyViewer.saveas") + "...");
   }

   @Override
   public List<PropertyPanel.PropertyCouple> getSpecificProperties() {
      String name;
      String binary = getModel().getBinary();
      if (binary == null) {
         name = "";
      } else {
         File file = new File(binary);
         name = file.getName();
      }

      if (name.length() == 0) {
         resetButtonText(_jbuttonSelectFile);
      } else {
         _jbuttonSelectFile.setText(name);
      }
      _jbuttonSelectFile.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            selectDataFile();
         }
      });
      ArrayList<PropertyPanel.PropertyCouple> properties = new ArrayList<>();
      properties.add(new PropertyPanel.PropertyCouple(ArianeSettings.getString("PropertyViewer.path"), _jbuttonSelectFile));
      return properties;
   }

   public void setButtonEnabled( boolean flag ) {
      _jbuttonSelectFile.setEnabled(flag);
   }

   ///// 
   ///// M E N U 
   /////
   @Override
   protected JPopupMenu initPopup() {
      JPopupMenu popupMenu = new JPopupMenu();

      popupMenu.add(createExecuteMenuItem());
      popupMenu.add(createResetMenuItem());

      popupMenu.addSeparator();

      popupMenu.add(createCopyMenuItem());
      popupMenu.add(createCutMenuItem());
      popupMenu.add(createDeleteMenuItem());

      popupMenu.addSeparator();

      popupMenu.add(createSelectFileItem());

      return popupMenu;
   }

   private JMenuItem createSelectFileItem() {
      JMenuItem item;
      item = new JMenuItem(ArianeSettings.getString("PopupMenu.selectFile"));
      item.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            selectDataFile();
         }
      });
      return item;
   }

   /**
    * This class is used to store the size and location of the previous opening of JFileChooser.
    */
   private static class StaticSaveFileChooser extends JFileChooser {

      private JDialog _dialog;

      StaticSaveFileChooser() {
         setDialogTitle(ArianeSettings.getString("FileChooser.titleSaveFile"));
         setDialogType(JFileChooser.SAVE_DIALOG);
      }

      @Override
      protected JDialog createDialog( Component parent ) {
         JDialog dialog = super.createDialog(parent);
         if (_dialog != null) {
            dialog.setLocation(_dialog.getLocation());
            dialog.setSize(_dialog.getSize());
         }
         _dialog = dialog;
         return dialog;
      }
   }
}
