/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph.item;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.model.exception.ArianeRuntimeException;
import fr.greyc.pantheon.ariane.model.graph.item.GraphItem.Status;
import fr.greyc.pantheon.ariane.model.graph.item.ViewerItem;
import fr.greyc.pantheon.ariane.presenter.command.ChangeViewerCommandEdit;
import fr.greyc.pantheon.ariane.view.PropertyPanel;

public final class ViewerItemView extends DataFileView {

   private final JTextField _commandTextfield = new JTextField();

   public ViewerItemView() {
      super("viewer");
   }

   @Override
   protected void resetButtonText( JButton button ) {
      // Do nothing
   }

   @Override
   protected void onMouseDoubleClicked( MouseEvent event ) {
      if (getModel().getExecutionStatus() == Status.EXECUTED) {
         displayDataFileContents();
      }
   }

   @Override
   public void changeBinaryName( JTextField textfield1, String binary, boolean addInHistory ) {
      // Empty.
   }

   public void changeCommandName( String newCommand, boolean addInHistory ) {
      final String oldCommand = getModel().getBinary();
      if (newCommand != null && newCommand.length() > 0 && !newCommand.equals(UserPreferences.getCommandVisu())) {
         getModel().setBinary(newCommand);
      } else {
         getModel().setBinary(null);
      }
      if (getModel().getBinary() == null) {
         _commandTextfield.setText(UserPreferences.getCommandVisu());
      } else {
         _commandTextfield.setText(getModel().getBinary());
      }
      getModel().getView().updatePropertyViewer();
      getGraph().setModified(true);
      getModel().resetAll();
      if (addInHistory) {
         getGraph().validateOneAction(new ChangeViewerCommandEdit((ViewerItem) getModel(), newCommand, oldCommand));
      }
   }

   ///// 
   ///// M E N U 
   /////
   @Override
   protected JPopupMenu initPopup() {
      JPopupMenu popupMenu = new JPopupMenu();

      popupMenu.add(createExecuteMenuItem());
      popupMenu.add(createResetMenuItem());

      popupMenu.addSeparator();

      popupMenu.add(createCopyMenuItem());
      popupMenu.add(createCutMenuItem());
      popupMenu.add(createDeleteMenuItem());

      return popupMenu;
   }

   @Override
   public List<PropertyPanel.PropertyCouple> getSpecificProperties() {
      if (getModel().getBinary() == null) {
         _commandTextfield.setText(UserPreferences.getCommandVisu());
      } else {
         _commandTextfield.setText(getModel().getBinary());
      }
      _commandTextfield.addFocusListener(new FocusListener() {
         @Override
         public void focusLost( FocusEvent event ) {
            changeCommandName(((JTextField) (event.getSource())).getText().trim(), true);
         }

         @Override
         public void focusGained( FocusEvent event ) {
            // Unused
         }
      });
      ArrayList<PropertyPanel.PropertyCouple> properties = new ArrayList<>();
      properties.add(new PropertyPanel.PropertyCouple(ArianeSettings.getString("PropertyViewer.commandName"), _commandTextfield));
      return properties;
   }
}
