/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph.slot;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.graph.slot.OutputDataSlot;
import fr.greyc.pantheon.ariane.view.graph.GraphView;

import java.awt.Color;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;

public final class InputDataSlotView extends SlotView {

   private static final transient ImageIcon CONNECTED_ICON;
   private static final transient ImageIcon DISCONNECTED_ICON;

   static {
      CONNECTED_ICON = ArianeSettings.getIcon("DataSlotOn.png");
      DISCONNECTED_ICON = ArianeSettings.getIcon("DataSlotOff.png");
   }

   public InputDataSlotView() {
      setIcon(DISCONNECTED_ICON);
   }

   @Override
   public void setConnectedIcon() {
      setIcon(CONNECTED_ICON);
   }

   @Override
   public void setDisconnectedIcon() {
      setIcon(DISCONNECTED_ICON);
   }

   @Override
   protected Color getLinkColor() {
      return GraphView.getImageLinkColor();
   }

   @Override
   public Point getAnchor() {
      return new Point((int) (getLocation().getX() - 3 + getModel().getGraphItem().getLocation().getX()),
          (int) (getLocation().getY() + getHeight() / 2 + getModel().getGraphItem().getLocation().getY()));
   }

   @Override
   public JComponent getValueEditor() {
      JButton button = new JButton(ArianeSettings.getString("OutSlotImage.display.backgroundText"));
      button.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed( ActionEvent event ) {
            ((OutputDataSlot) getModel().getConnectedSlots().get(0)).showDataFileContents();
         }
      });
      return button;
   }
}
