/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.utils;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import javax.swing.ImageIcon;
import javax.swing.JButton;

import fr.greyc.pantheon.ariane.model.graph.slot.OutputDataSlot;

import fr.greyc.pantheon.imageio.ImageDecoder;
import fr.greyc.pantheon.swingext.ImageFilePreviewer;

/**
 * Displays a button with an image that can be pressed.
 * The image is either a text or an image depending on the output file contents.
 */
public final class ImageButton extends JButton {

   private final String _backgroundText;
   private final OutputDataSlot _slot;
   private static final int MAX_HEIGHT = 80;
   private static final int OFFSET = 8;

   public ImageButton( OutputDataSlot slot, String text ) {
      super(text);
      _backgroundText = text;
      _slot = slot;
      super.setMinimumSize(new Dimension(0, MAX_HEIGHT + OFFSET));
      super.setPreferredSize(new Dimension(0, MAX_HEIGHT + OFFSET));
   }

   @Override
   public void setEnabled( boolean enabled ) {
      if (enabled) {
         new ImageFileReader().start();
      } else {
         setIcon(null);
         setText(_backgroundText);
         setHorizontalAlignment(CENTER);
      }
      super.setEnabled(enabled);
   }

   protected ImageIcon getThumbnail( String filename ) {
      File file = new File(filename);
      if (!file.canRead()) {
         return null;
      } else if (ImageFilePreviewer.isText(file, false)) {
         return readTextFile(file);
      } else {
         return ImageDecoder.getThumbnail(filename, MAX_HEIGHT);
      }
   }

   private ImageIcon readTextFile( File file ) {
      try (FileInputStream stream = new FileInputStream(file)) {
         byte[] data = new byte[300];
         int bytesRead = stream.read(data);
         if (bytesRead <= 0) {
            throw new IOException("Invalid file");
         }
         String text = new String(data, 0, bytesRead);
         String[] lines = text.split("\n");
         int width = 200;
         BufferedImage img = new BufferedImage(width, MAX_HEIGHT, BufferedImage.TYPE_INT_ARGB);
         Graphics2D g2d = img.createGraphics();
         g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
         g2d.setFont(g2d.getFont().deriveFont(9.0F));
         final int yoffset = 9;
         g2d.setColor(getForeground());
         for (int i = 0; i < lines.length; i++) {
            g2d.drawString(lines[i], 0, yoffset * (i + 1));
         }
         ImageIcon icon = new ImageIcon(img);
         setHorizontalAlignment(LEFT);
         return icon;
      } catch (IOException e) {
         return null;
      }
   }

   private class ImageFileReader extends Thread {

      @Override
      public void run() {
         ImageIcon icon = getThumbnail(_slot.getValue());
         if (icon != null) {
            setIcon(icon);
            setText(null);
            setHorizontalAlignment(CENTER);
         } else {
            setIcon(null);
            setText(_backgroundText);
            setHorizontalAlignment(CENTER);
         }
      }
   }
}
