/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.utils;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.plaf.basic.BasicFileChooserUI;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.swingext.MessageBox;

/**
 * A dialog box for selecting the script filename.
 */
public final class ScriptNameChooser {

   private ScriptNameChooser() {
   }

   public static String[] chooseFilename( final String graphName, final String[][] knownScriptNames ) {
      String[] chosenFile = getFileNameFromFileChooser(graphName, knownScriptNames);
      if (chosenFile.length > 0) {
         String filename = chosenFile[0];
         File file = new File(filename);
         String[] options = {ArianeSettings.getString("overwriteButtonText"),
            ArianeSettings.getString("nooverwriteButtonText")};
         if (file.exists() && MessageBox.showConfirmDialog(Ariane.getFrame(),
             ArianeSettings.getString("Generator.dialog.message.part1")
             + filename
             + ArianeSettings.getString("Generator.dialog.message.part2"),
             ArianeSettings.getString("Generator.dialog.title"),
             options, options[0]) != 0) {
            return new String[0];
         }
         return chosenFile;
      }
      return new String[0];
   }

   private static String[] getFileNameFromFileChooser( String filebasename, String[][] scriptNames ) {
      File currentDirName = new File(UserPreferences.getCurrentDirectory());
      final JFileChooser fileChooser = new JFileChooser();
      fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
      fileChooser.setCurrentDirectory(currentDirName);
      fileChooser.setDialogTitle(ArianeSettings.getString("Generator.dialogchooser.title"));
      fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
      fileChooser.addPropertyChangeListener(new PropertyChangeListener() {
         @Override
         public void propertyChange( final PropertyChangeEvent event ) {
            if (JFileChooser.FILE_FILTER_CHANGED_PROPERTY.equals(event.getPropertyName())) {
               updateScriptNameFromExtension(fileChooser, event);
            }
         }
      });

      // Use file type preference
      FileNameExtensionFilter defaultFilter = null;
      final int typePreference = UserPreferences.getCodeGeneratorIndex();
      fileChooser.setAcceptAllFileFilterUsed(false);
      for (int i = 0; i < scriptNames.length; i++) {
         FileNameExtensionFilter filter = new FileNameExtensionFilter(scriptNames[i][1], scriptNames[i][2]);
         fileChooser.addChoosableFileFilter(filter);
         if (i == typePreference) {
            defaultFilter = filter;
         }
      }
      fileChooser.setFileFilter(defaultFilter);
      // Set the first name using default extension
      String extension = defaultFilter.getExtensions()[0];
      File currentFileName = new File(filebasename + "." + extension);
      fileChooser.setSelectedFile(currentFileName);

      if (fileChooser.showSaveDialog(Ariane.getFrame()) == JFileChooser.APPROVE_OPTION) {
         // Update file type preference.
         for (int i = 0; i < scriptNames.length; i++) {
            if (scriptNames[i][1].equals(fileChooser.getFileFilter().getDescription())) {
               UserPreferences.getCodeGeneratorIndex(i);
            }
         }

         File theFile = fileChooser.getSelectedFile();
         if (theFile != null) {
            String fullFileName = fileChooser.getSelectedFile().getPath();
            UserPreferences.setCurrentDirectory(fileChooser.getSelectedFile().getParent());
            String description = ((FileNameExtensionFilter) fileChooser.getFileFilter()).getDescription();
            return new String[]{fullFileName, description};
         }
      }
      return new String[0];
   }

   private static void updateScriptNameFromExtension( final JFileChooser fileChooser, PropertyChangeEvent event ) {
      JFileChooser chooser = (JFileChooser) event.getSource();
      FileNameExtensionFilter filename = (FileNameExtensionFilter) chooser.getFileFilter();
      if (filename != null) {
         String currentName = ((BasicFileChooserUI) fileChooser.getUI()).getFileName();
         if (currentName.lastIndexOf('.') > 0) {
            currentName = currentName.substring(0, currentName.lastIndexOf('.'));
         }
         final String extension = ((FileNameExtensionFilter) fileChooser.getFileFilter()).getExtensions()[0];
         final String fullFileName = currentName + "." + extension;
         fileChooser.setSelectedFile(new File(fullFileName));
      }
   }
}
