/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.swingext;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.Timer;

/**
 * Implementation of fade in and fade out effects inspired from gamification.
 * This is a static class that defines the methods : fadeIn() and fadeOut().
 * Example of use:
 * FadingEffects.fadeIn(frame);
 * JOptionPane.showMessageDialog(frame, "alert", "alert", JOptionPane.ERROR_MESSAGE);
 * FadingEffects.fadeOut(frame);
 */
public final class FadingEffects {

   private static final int DEFAULT_DELAY = 40;
   private static final int DEFAULT_ITERATIONS = 10;
   private static final float MAX_OPACITY = 0.4f;

   private static Component _glassPane;
   private static int _delay;
   private static float _iterationIncrement;

   private FadingEffects() {
   }

   public static void fadeIn( final JFrame frame ) {
      fadeIn(frame, DEFAULT_DELAY, DEFAULT_ITERATIONS);
   }

   private static void fadeIn( final JFrame frame, final int delay, final int iterations ) {
      _delay = delay;
      _iterationIncrement = MAX_OPACITY / iterations;
      _glassPane = frame.getGlassPane();
      final FadingMirror glassPane = new FadingMirror();
      frame.setGlassPane(glassPane);
      glassPane.setVisible(true);
      glassPane.beginFade();
   }

   public static void fadeOut( final JFrame frame ) {
      frame.getGlassPane().setVisible(false);
      frame.setGlassPane(_glassPane);
   }

   private static final class FadingMirror extends JPanel implements ActionListener {

      private float _opacity;
      private Timer _fadeTimer;

      FadingMirror() {
         super.setBackground(Color.BLACK);
         super.setOpaque(false);
      }

      public void beginFade() {
         _fadeTimer = new Timer(_delay, this);
         _fadeTimer.setInitialDelay(0);
         _fadeTimer.start();
      }

      @Override
      public void actionPerformed( final ActionEvent event ) {
         _opacity += _iterationIncrement;
         if (_opacity > MAX_OPACITY) {
            _opacity = MAX_OPACITY;
            _fadeTimer.stop();
         }
         repaint();
      }

      @Override
      public void paintComponent( final Graphics graphics ) {
         ((Graphics2D) graphics).setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, _opacity));
         graphics.setColor(getBackground());
         graphics.fillRect(0, 0, getWidth(), getHeight());
      }
   }
}
