/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.swingext.multivieweditor;

import java.awt.Component;
import java.awt.Point;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DragSource;
import java.io.IOException;

import javax.activation.ActivationDataFlavor;
import javax.activation.DataHandler;
import javax.swing.JComponent;
import javax.swing.TransferHandler;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.tree.OperatorTreeItem;
import fr.greyc.pantheon.ariane.model.tree.SubroutineTreeItem;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;

public final class TabTransferHandler extends TransferHandler {

   private final DataFlavor _localObjectFlavor;
   private DnDTabbedPane _source;
   private static GhostGlassPane _glassPane;

   public TabTransferHandler() {
      _localObjectFlavor = new ActivationDataFlavor(DnDTabbedPane.class, DataFlavor.javaJVMLocalObjectMimeType, "DnDTabbedPane");
   }

   @Override
   protected Transferable createTransferable( JComponent component ) {
      if (component instanceof DnDTabbedPane) {
         _source = (DnDTabbedPane) component;
      }
      return new DataHandler(component, _localObjectFlavor.getMimeType());
   }

   @Override
   public int getSourceActions( JComponent component ) {
      DnDTabbedPane src = (DnDTabbedPane) component;
      if (_glassPane == null) {
         _glassPane = new GhostGlassPane(src);
         component.getRootPane().setGlassPane(_glassPane);
      }
      if (src.getDragTabIndex() < 0) {
         return TransferHandler.NONE;
      }
      component.getRootPane().getGlassPane().setVisible(true);
      return TransferHandler.MOVE;
   }

   @Override
   public boolean canImport( TransferHandler.TransferSupport support ) {
      if (!support.isDrop()) {
         return false;
      } else if (support.isDataFlavorSupported(OperatorTreeItem.FLAVORS[0])) {
         return true;
      } else if (support.isDataFlavorSupported(SubroutineTreeItem.FLAVORS[0])) {
         return true;
      } else if (support.isDataFlavorSupported(_localObjectFlavor)) {
         support.setDropAction(TransferHandler.MOVE);
         TransferHandler.DropLocation tdl = support.getDropLocation();
         Point pt = tdl.getDropPoint();
         DnDTabbedPane target = (DnDTabbedPane) support.getComponent();
         target.autoScrollTabList(pt);
         DnDTabbedPane.DropLocation dl = target.calculateDropLocationForPoint(pt);
         final int idx = dl.getTabIndex();
         boolean isDropable;
         if (target.equals(_source)) {
            isDropable = target.getTabAreaBounds().contains(pt) && idx >= 0 && idx != target.getDragTabIndex() && idx != target.getDragTabIndex() + 1;
         } else if (_source != null && target != _source.getComponentAt(_source.getDragTabIndex())) {
            isDropable = target.getTabAreaBounds().contains(pt) && idx >= 0;
         } else {
            isDropable = false;
         }

         Component glassPane = target.getRootPane().getGlassPane();
         glassPane.setCursor(isDropable ? DragSource.DefaultMoveDrop : DragSource.DefaultMoveNoDrop);

         if (isDropable) {
            support.setShowDropLocation(true);
            dl.setDropable(true);
            target.initDropLocation(dl, null, true);
            return true;
         } else {
            support.setShowDropLocation(false);
            dl.setDropable(false);
            target.initDropLocation(dl, null, false);
            return false;
         }
      } else {
         return false;
      }
   }

   @Override
   public boolean importData( TransferHandler.TransferSupport support ) {
      TreeItem treeItem;
      if (support.isDataFlavorSupported(OperatorTreeItem.FLAVORS[0])) {
         try {
            treeItem = (TreeItem) support.getTransferable().getTransferData(OperatorTreeItem.FLAVORS[0]);
            DnDTabbedPane target = (DnDTabbedPane) support.getComponent();
            return addNewItem(support, treeItem, target);
         } catch (UnsupportedFlavorException | IOException e) {
            return false;
         }
      } else if (support.isDataFlavorSupported(SubroutineTreeItem.FLAVORS[0])) {
         try {
            treeItem = (TreeItem) support.getTransferable().getTransferData(SubroutineTreeItem.FLAVORS[0]);
            DnDTabbedPane target = (DnDTabbedPane) support.getComponent();
            return addNewItem(support, treeItem, target);
         } catch (UnsupportedFlavorException | IOException e) {
            return false;
         }
      } else if (support.isDataFlavorSupported(_localObjectFlavor)) {
         return moveTab(support);
      } else {
         return false;
      }
   }

   @Override
   protected void exportDone( JComponent src, Transferable data, int action ) {
      _glassPane.setVisible(false);
      _glassPane = null;
      _source = null;
   }

   private boolean addNewItem( TransferHandler.TransferSupport support, TreeItem treeItem, DnDTabbedPane target ) {
      if (target.getTabCount() == 0 && target.isRightTabbedPane()) {
         Point position = support.getDropLocation().getDropPoint();
         Ariane.getPresenter().getWorkspace().createNewGraph().addItem(treeItem, position);
         return true;
      }
      return false;
   }

   private boolean moveTab( TransferHandler.TransferSupport support ) {
      try {
         DnDTabbedPane target = (DnDTabbedPane) support.getComponent();
         DnDTabbedPane source = (DnDTabbedPane) support.getTransferable().getTransferData(_localObjectFlavor);
         DnDTabbedPane.DropLocation dl = target.getDropLocation();
         int index = dl.getTabIndex();
         if (target.equals(source)) {
            source.convertTab(source.getDragTabIndex(), index);
         } else {
            source.exportTab(source.getDragTabIndex(), target, index);
         }
         return true;
      } catch (UnsupportedFlavorException | IOException ioe) {
      }
      return false;
   }
}
